@@ -4,12 +4,12 @@ pub mod core {
     /// Contains mutable simulation state which can be changed via callback functions
     #[derive(Copy, Clone)]
     pub struct State {
-        update_interval: u32,
         timescale: f32,
         simulate: bool,
         clock_start: Instant,
         last_tick: Instant,
         delta_time: u32,
+        timestep: f32,
         irl_time: Duration,
         sim_time: Duration
     }
@@ -18,30 +18,32 @@ pub mod core {
         /// Creates a default State object
         pub fn new() -> State {
             // Create default state object
-            let mut new_state = State {
-                update_interval: 40,
+            let new_state = State {
                 timescale: 1.0,
                 simulate: true,
                 clock_start: Instant::now(),
                 last_tick: Instant::now(),
                 delta_time: 0,
+                timestep: 0.0,
                 irl_time: Duration::new(0,0),
                 sim_time: Duration::new(0,0)
             };
 
-            // Make sure that delta_time always starts the same as update_interval
-            new_state.delta_time = new_state.update_interval;
-
             // Return this default state
             new_state
         }
 
-        /// Returns the current "delta time", the time elapsed since the last update tick in milliseconds
+        /// Returns the current "delta time", the real time (in ms) elapsed since the last update tick
         pub fn get_delta_time(self) -> u32 {
             self.delta_time
         }
 
-        /// Returns the current IRL time elapsed since the start of the simulation
+        /// Returns the current "timestep", the virtual time (in s) elapsed since the last update tick (necessary for scaling physics simulations, etc.)
+        pub fn get_timestep(self) -> f32 {
+            self.timestep
+        }
+
+        /// Returns the current real time elapsed since the start of the simulation
         pub fn get_irl_time(self) -> Duration {
             self.irl_time
         }
@@ -76,32 +78,51 @@ pub mod core {
             self.timescale = timescale;
         }
 
-        /// Prints a string of information about the current time (IRL time, Sim time, Delta time (tick), Delta time (step))
-        /// IRL time: Real time (in ms) elapsed since the start of the loop
-        /// Sim time: Virtual time (in ms) elapsed since the start of the loop
-        /// Delta time (tick): Real time (in ms) elapsed between the last tick and the previous tick
-        /// Delta time (step): Real time (in ms with nanosecond accuracy) elapsed since the last update tick
+        /// Prints a string of information about the current step's timings
+        ///
+        /// # Example:
+        /// `IRL time: 4443ms | Sim time: 4443ms | Delta time (tick): 40ms | Delta time (step): 40.0638ms | Timestep: 0.04s`
+        /// # Terminology:
+        /// - *IRL time:* Real time (in ms) elapsed since the start of the simulation
+        /// - *Sim time:* Virtual time (in ms) elapsed since the start of the simulation
+        /// - *Delta time (tick):* Real time (in ms) elapsed between the last tick and the previous tick
+        /// - *Delta time (step):* Real time (in ms with ns accuracy) elapsed since the last tick
+        /// - *Timestep:* Virtual time (in s with ms accuracy) elapsed since the last tick
         pub fn debug_time(self) {
             let elapsed_time = Instant::now().duration_since(self.last_tick);
             let loop_delay_ms = elapsed_time.as_nanos() as f32 / 1_000_000.0;
-            println!("IRL time: {}ms | Sim time: {}ms | Delta time (tick): {}ms | Delta time (step): {}ms", self.irl_time.as_millis(), self.sim_time.as_millis(), self.delta_time, loop_delay_ms);
+            println!("IRL time: {}ms | Sim time: {}ms | Delta time (tick): {}ms | Delta time (step): {}ms | Timestep: {}s", self.irl_time.as_millis(), self.sim_time.as_millis(), self.delta_time, loop_delay_ms, self.timestep);
         }
     }
 
     /// The simulation loop itself
     pub struct Loop {
         state: State,
-        realtime: bool
+        realtime: bool,
+        update_interval: u32
     }
     
     impl Loop {
         /// Creates a new simulation with default values
         pub fn new() -> Loop {
-            // Return a Loop object with a default State
-            Loop {
-                state: State::new(),
-                realtime: true
-            }
+            // Create a new State object
+            let mut new_state = State::new();
+            
+            // Create a Loop object with a default State
+            let mut new_loop = Loop {
+                state: new_state,
+                realtime: true,
+                update_interval: 40
+            };
+            
+            // Initialize the delta time to be the same as the update interval (to prevent division by zero)
+            new_loop.state.delta_time = new_loop.update_interval;
+
+            // Initialize the timestep based on the new delta time
+            new_loop.state.timestep = timestep(new_loop.state.delta_time, new_loop.state.timescale);
+
+            // Return the now-initialized Loop
+            new_loop
         }
     
         /// Initializes or re-initializes the simulation
@@ -116,11 +137,11 @@ pub mod core {
         }
 
         /// Executes the per-loop logic (can be triggered manually so that hypoloop can be tied into external event loops)
-        pub fn step(&mut self, mut update_callback: impl FnMut(&mut State), display_callback: impl Fn(&State)) {
+        pub fn step(&mut self, mut update_callback: impl FnMut(&mut State), mut display_callback: impl FnMut(&mut State)) {
             // don't run if the simulation is paused
             if self.state.simulate {
                 // TODO - support frameskips
-                if !self.realtime || delta_time(self.state.last_tick) >= self.state.update_interval {
+                if !self.realtime || delta_time(self.state.last_tick) >= self.update_interval {
                     // mutable delta time and timescale for flexibility
                     let elapsed_time = Instant::now().duration_since(self.state.last_tick);
                     
@@ -130,10 +151,11 @@ pub mod core {
                         self.state.sim_time += elapsed_time.mul_f32(self.state.timescale);
                         self.state.irl_time += elapsed_time;
                     } else {
-                        self.state.delta_time = self.state.update_interval;
-                        self.state.sim_time += Duration::from_millis(self.state.update_interval as u64);
+                        self.state.delta_time = self.update_interval;
+                        self.state.sim_time += Duration::from_millis(self.update_interval as u64);
                         self.state.irl_time = Instant::now().duration_since(self.state.clock_start);
                     }
+                    self.state.timestep = timestep(self.state.delta_time, self.state.timescale);
         
                     // update
                     update_callback(&mut self.state);
@@ -144,7 +166,7 @@ pub mod core {
         
                 // display
                 if self.realtime {
-                    display_callback(&self.state);
+                    display_callback(&mut self.state);
                 }
             }
         }
@@ -153,10 +175,25 @@ pub mod core {
         pub fn set_realtime(&mut self, realtime: bool) {
             self.realtime = realtime;
         }
+
+        /// Returns the "update interval", the minimum time (in ms) which will elapse between update ticks
+        pub fn get_update_interval(self) -> u32 {
+            self.update_interval
+        }
+
+        /// Changes the update interval
+        pub fn set_update_interval(&mut self, update_interval: u32) {
+            self.update_interval = update_interval;
+        }
     }
     
-    // gets the time in milliseconds that's elapsed since the earlier Instant
+    // gets the real time (in ms) that's elapsed since the earlier Instant
     fn delta_time(earlier: Instant) -> u32 {
         Instant::now().duration_since(earlier).as_millis() as u32
     }
+
+    // returns the fractional timestep (in s) based on delta time and timescale
+    fn timestep(delta_time: u32, timescale: f32) -> f32 {
+        delta_time as f32 / 1000.0 * timescale
+    }
 }
\ No newline at end of file
